--_________________________________________________________________________________________________
--                                                                                                |
--                                         |The nanoFIP|                                          |
--                                                                                                |
--                                        CERN,BE/CEM-EDL                                         |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                       wf_trst_controller                                       |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         wf_trst_controller.vhd                                                            |
--                                                                                                |
-- Description  After the reception and validation of a consumed var_4 RP_DAT frame, the unit     |
--              is responsible for driving the "nanoFIP, User Interface, JTAG TRST Controller"    |
--              signal JC_TRST_N which is the reset of the JTAG TAP copntroller.                  |
--                                                                                                |
--                o JC_TRST should be asserted in order to allow reprogramming of the Radiation   |
--                  tolerant DI/OT System Board through worldfip.                                 |
--                                                                                                |
--                                                                                                |
-- Authors      Konstantinos Blantos <Konstantinos.Blantos@cern.ch>                               |
-- Date         03/2024                                                                           |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                                      SOLDERPAD LICENSE                                         |
--                                   Copyright CERN 2014-2018                                     |
--                              ------------------------------------                              |
-- Copyright and related rights are licensed under the Solderpad Hardware License, Version 2.0    |
-- (the "License"); you may not use this file except in compliance with the License.              |
-- You may obtain a copy of the License at http://solderpad.org/licenses/SHL-2.0.                 |
-- Unless required by applicable law or agreed to in writing, software, hardware and materials    |
-- distributed under this License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR       |
-- CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language   |
-- governing permissions and limitations under the License.                                       |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                       Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all; -- std_logic definitions
use IEEE.NUMERIC_STD.all;    -- conversion functions
-- Specific library
library work;
use work.WF_PACKAGE.all;     -- definitions of types, constants, entities


--=================================================================================================
--                            Entity declaration for wf_jtag_controller
--=================================================================================================
entity wf_trst_controller is port(
  -- INPUTS
    -- nanoFIP User Interface, General signal
    uclk_i          : in std_logic;                      -- 40 MHz clock

    -- Signal from the wf_reset_unit
    nfip_rst_i      : in std_logic;                      -- nanoFIP internal reset

    -- Signals from the wf_engine_control unit
    var_id_i        : in t_var;

  -- OUTPUTS
    -- nanoFIP User Interface, JTAG TRST Controller signal
    jc_trst_o       : out std_logic);                    -- JTAG TRST output

end entity wf_trst_controller;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture rtl of wf_trst_controller is

    -- Internal JTAG TRST signal
    signal s_jc_trst    : std_logic;
    -- Internal var_id signal
    signal s_var        : t_var;
 
--=================================================================================================
--                                       architecture begin
--=================================================================================================
begin

  -- Assign var_id
  s_var <= var_id_i;

  -- Assign output JTAG TRST signal
  JC_TRST_Controller : process(uclk_i)
  begin
    if rising_edge(uclk_i) then
      if nfip_rst_i = '1' then
        s_jc_trst <= '0';
      elsif (s_var = var_4 or s_var = var_5) then
        s_jc_trst <= '1';
      elsif (s_var = var_rst) then
        s_jc_trst <= '0';
      end if;
    end if;
  end process JC_TRST_Controller;

  jc_trst_o <= s_jc_trst;

end architecture rtl;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------